//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
#include "Drawable.hpp"
//------------------------------------------------------------------------------
#include "Group.hpp"
//------------------------------------------------------------------------------
#include "MyException.hpp"
//------------------------------------------------------------------------------
#include <iostream>
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
using namespace std;
//------------------------------------------------------------------------------
Drawable::Drawable(Group *parent, const std::string &name) : Object(parent, name), m_isVisible(true), m_zpos(0), m_lowZpos(0)
{
    m_parent = NULL;
    ChangeParent(parent);
}

//------------------------------------------------------------------------------
void Drawable::Show(bool visible)
{
    m_isVisible = visible;
}
//------------------------------------------------------------------------------
void Drawable::Hide()
{
    Show(false);
}
//------------------------------------------------------------------------------
bool Drawable::IsVisible()
{
    return m_isVisible;
}
//------------------------------------------------------------------------------
/*Drawable::Drawable(const Drawable& src) : Object(src)
{
    m_zpos = src.m_zpos;
    m_isVisible = src.m_isVisible;
    GetParentRaw()->Add(this);
}*/
//------------------------------------------------------------------------------
void Drawable::RegisterClone(Group *newParent)
{
    // We change the parent if it's needed
    if(newParent != NULL)
    {
        m_parent = newParent;
    }

    // /!\ Specific to Drawable object : we need to refresh the lowPos
    m_lowZpos = GetParentRaw()->Add(this);

    // We notice the object that the parent has changed => Will attach the objet to the pysic world and / or its childs
    OnChangeScreen(NULL, GetParentScreenRaw());
}
//------------------------------------------------------------------------------
void Drawable::ChangeParent(Group *parent)
{
    // /!\ à copier pour PhysicObject quant modifié !!
    Screen *backScreen = NULL, *newScreen = NULL;

    if(m_parent != NULL)
    {
        backScreen = GetParentScreenRaw();
        m_parent->Remove(this);
    }

    // m_parent = parent; // If parent is NULL, the object wille be deleted and it may need to know its parent

    if(parent != NULL)
    {
        m_parent = parent;
        m_lowZpos = parent->Add(this);
        newScreen = GetParentScreenRaw();
    }

    OnChangeScreen(backScreen, newScreen);
}
//------------------------------------------------------------------------------
void Drawable::SetZpos(int zpos)
{
    if(GetParentRaw() != NULL)
    {
        // We add remove and add the object from its parent to change the map in witch it is registrated
        GetParentRaw()->Remove(this);
        m_zpos = zpos;
        m_lowZpos = GetParentRaw()->Add(this);
    }
    else
    {
        MY_THROW("SetZPos but no parent !"); // Zpos sans parent => pas de sens
    }
}
//------------------------------------------------------------------------------
int Drawable::GetZpos() const
{
    return m_zpos;
}
//------------------------------------------------------------------------------
/** Get the low pos (the low zPos is a creation order, for the object that have the same zPos) */
int Drawable::GetLowZPos() const
{
    return m_lowZpos;
}
//------------------------------------------------------------------------------
/** Return true if this Drawable is on to of the the given one (higher Zpos) */
bool Drawable::IsUpperThan(const Drawable *drawableToCompare) const
{
    return (GetZpos() > drawableToCompare->GetZpos() || (GetZpos() == drawableToCompare->GetZpos() && (GetLowZPos() > drawableToCompare->GetLowZPos())));
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
DrawableSF::DrawableSF(Group *parent, const std::string &name) : Drawable(parent, name)
{

}
//------------------------------------------------------------------------------
void DrawableSF::RenderSf(sf::RenderTarget &App, sf::Drawable *drawable)
{
    // /!\ Optimisation
    // FIXME: Ne pas calculer GetGlobalPosition et GetGlobalRotation à chaque fois : passer et paramétre la roation et position globale du groupe parent !
    if(IsVisible())
    {
        drawable->SetPosition(GetGlobalPosition());
        drawable->SetRotation(GetGlobalRotation() / (2 * M_PI) * 360.f);
        App.Draw(*drawable);
    }
}
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
