//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef TEXTURED_AREA_HPP
#define TEXTURED_AREA_HPP

#include "Sprite.hpp"

#include <SFML/Graphics.hpp>
//------------------------------------------------------------------------------
namespace En
{
/// LUA_HEADER_COPY_ON ///
/** A TexturedArea is used to render a texture. It can be resised. To create a Shape, you should use Group::CreateShape. */
/// LUA_HEADER_COPY_OFF ///

class TexturedArea : public Sprite
{
public:
    TexturedArea(Group *parent, const std::string &name);
    static TexturedArea *Create(Group *parent, const std::string &name) {return new TexturedArea(parent, name);}

    /// LUA_EXPORT_ON(TexturedArea, Sprite, ProtectedPtr<Object>) ///

    /** Change the area size. */
    //LUA// luaProperty(Vector2D, size, GetAreaSize, SetAreaSize);

    /** Get the current size of the textured area */
    Vector2D GetAreaSize() const;

    /** Set the current size of the area */
    void SetAreaSize(const Vector2D &newSize);


    /** Change the area size. */
    //LUA// luaProperty(Vector2D, textureSpeed, SetTextureSpeed, GetTextureSpeed);

    /** Set the texture speed. This could be used to easily animate a textured aread. */
    void SetTextureSpeed(const Vector2D &speed);

    /** Get the texture speed. This could be used to easily animate a textured aread. */
    Vector2D GetTextureSpeed() const;


    /** Change the offset of the texture. */
    //LUA// luaProperty(Vector2D, textureOffset, SetTextureOffset, GetTextureOffset);

    /** Set the offset of the texture. */
    void SetTextureOffset(const Vector2D &offset);

    /** Get the offset of the texture. */
    Vector2D GetTextureOffset() const;

    /// LUA_EXPORT_OFF ///

    // No Lua
    /** Create a clone of the object, change the parent and return an handler to the clone. */
    virtual TexturedArea *GetRawCloneToGroup(Group *newParent) const {TexturedArea *obj = new TexturedArea(*this); obj->RegisterClone(newParent); return obj;}

    // Render
    virtual void Render(sf::RenderTarget &App);

    // Change the texute offset
    virtual void ProceedMvt(float time);

    // Type
    ADD_GET_TYPE_FN(OBJ_TYPE_DRAWABLE_TEXTURED_AREA)
protected:
    Vector2D m_areaSize, m_textureSpeed, m_textureOffset;
};
//------------------------------------------------------------------------------
} // namespace En

#endif // TEXTURED_AREA_HPP
